#![windows_subsystem = "windows"]
// La primera línea es una directiva para que Rust no deje el ejecutable corriendo en la consola.
// Si queréis hacer experimentos e imprimir variables con println podéis comentar esta línea y quedará la consola
// mientras esté abierto el programa.

extern crate native_windows_derive as nwd;
extern crate native_windows_gui as nwg;
use nwd::NwgUi;
use nwg::NativeUi;
// Estas líneas le están diciendo a Rust: vamos a usar nwg y nwd.

use std::cell::RefCell;
// RefCell es un tipo de datos que permite mutabilidad interior.

#[derive(Default, NwgUi)]
pub struct BasicApp {
    #[nwg_control(title: "Ejemplo básico", flags: "WINDOW|VISIBLE")]
    #[nwg_events( OnWindowClose: [BasicApp::say_goodbye] )]
    window: nwg::Window,
    // Esta es la ventana principal. Como veis tiene unos atributos como el título, visibilidad, etc.
    // También despacha un evento. Cuando se cierra, llama a la función say_goodbye.

    #[nwg_control(text: "Nombre", focus: true,)]
    name_edit: nwg::TextInput,
    // Este es un cuadro de edición precompletado con "nombre" y que adquiere el foco.

    #[nwg_control(text: "Di mi nombre.", )]
    #[nwg_events( OnButtonClick: [BasicApp::say_hello] )]
    hello_button: nwg::Button,
    // Aquí tenemos un botón, con sus atributos, y también se despacha un evento.
    // Al activarlo se llama a la función say_hello.

    #[nwg_control(text: "Bienvenido a la aplicación.", )]
    status_bar: nwg::StatusBar,
    // Barra de estado con un texto predefinido.

    clicks: RefCell<u32>,
    // Este es un RefCell, una célula que permite mutabilidad interior.
    // Contiene un elemento de tipo u32, que es un número sin signo de 32 bits.
    // Lo utilizaremos para contar el número de veces que se ha activado el botón.
    // En general, si necesitáis guardar datos lo mejor es que lo hagáis con RefCell.
}

// Ahora la implementación de las funciones de esta estructura:
impl BasicApp {
    fn say_hello(&self) {
        // La función toma &self, que es una referencia a la estructura para la que estamos implementando.

        nwg::simple_message("Hola", &format!("Hola {}", self.name_edit.text()));
        // Esto nos saca un cuadro de diálogo con un botón de aceptar, y el mensaje que sale ahí.
        // Fijaos en que se utiliza self-name_edit, que es el cuadro de edición, y su atributo text, para meter el nombre que hayamos puesto.

        self.name_edit.set_selection(0..self.name_edit.len());
        // Nos aseguramos que el texto en el cuadro de edición está seleccionado tras pulsar el botón.

        self.increment();
        // Llamamos a la función de la estructura para incrementar el número de clicks.

        self.status_bar.set_text(
            0,
            format!(
                "Ya has probado la aplicación {} veces.",
                self.clicks.borrow()
            )
            .as_str(),
        );
        // Ponemos el número de clicks (ya incrementado) en la barra de estado.

        self.hello_button.set_focus();
        // Por último, nos aseguramos que el foco permanece en el botón.
    }

    fn say_goodbye(&self) {
        nwg::simple_message("Adiós", &format!("Adiós, {}", self.name_edit.text()));
        // El mismo tipo de cuadro de diálogo.

        nwg::stop_thread_dispatch();
        // Esta es la forma de desmantelar el bucle principal del GUI para liberar recursos.
    }

    fn increment(&self) {
        // En esta función vamos a incrementar un número, clicks, que está en una RefCell.
        //  Por si sola no hace nada más, y la llama say-hello.

        let mut current = self.clicks.borrow_mut();
        // Aquí cogemos el valor de clicks con un préstamo mutable.

        *current += 1;
        // Y aquí lo incrementamos por referencia.
    }
}

fn main() {
    // Función principal.
    // Ten en cuenta que nwg corre en un hilo aparte su bucle principal. por eso aquí hay muy poca cosa.

    nwg::init().expect("Failed to init Native Windows GUI");
    // Esta es la inicialización del GUI. Si no tira es que el sistema está totalmente jodido y nos morimos.

    // Vamos a crear nuestro objeto de aplicación.
    let _app = BasicApp {
        clicks: 0.into(),
        // Inicializamos clicks a 0, utilizamos into porque es una RefCell.
        ..Default::default() // Y el resto a valores por defecto.
    };

    let _app = BasicApp::build_ui(Default::default()).expect("Failed to build UI");
    // Aquí utilizamos la maquinaria para construir el GUI a partir de nuestra estructura de datos.

    _app.name_edit.set_selection(0.._app.name_edit.len());
    // Nos aseguramos que el cuadro de edición tenga el texto seleccionado, porque me gusta más así.

    nwg::dispatch_thread_events();
    // Y aquí lanzamos el bucle principal de nwg a trabajar.
}
